from datetime import datetime

from aq_lib.event.event_type import EventType
from aq_lib.model.md_entry import MDEntry
from aq_lib.transport.serializable import Serializable


class Event(Serializable):
    def __init__(self, topic):
        self.topic = topic


# Event sent by cerebro to subscribe to MD
class MarketDataRequestEvent(Event):
    def __init__(self, topic=EventType.MARKET_DATA_REQUEST_EVENT.value, on_behalf_of_comp_id=None,
                 deliver_to_comp_id=None, symbols=None, md_req_id=None, subscription_request_type=None,
                 settlement_type=None, security_type=None, settlement_date=None, security_id=None, md_type=None,
                 routing_key=None, currency=None, settlement=None, market=None, description=None, execution_id=None,
                 negotiation_agent_id=None, trading_session=None, maturity_date=None, **kwargs):
        Event.__init__(self, topic)
        self.on_behalf_of_comp_id = on_behalf_of_comp_id
        self.deliver_to_comp_id = deliver_to_comp_id
        self.symbols = symbols
        self.md_req_id = md_req_id
        self.subscription_request_type = subscription_request_type
        self.routing_key = routing_key
        self.md_type = md_type
        self.settlement = settlement
        self.currency = currency
        self.security_id = security_id
        self.security_type = security_type
        self.settlement_date = settlement_date
        self.market = market
        self.description = description
        self.execution_id = execution_id
        self.negotiation_agent_id = negotiation_agent_id
        self.trading_session = trading_session
        self.maturity_date = maturity_date


class MarketDataSnapshotFullRefreshEvent(Event):
    def __init__(self, topic=EventType.MARKET_DATA_SNAPSHOT_FULL_REFRESH_EVENT.value, mdReqId=None, symbol=None,
                 md_type=None, securityId=None, securityType=None, settlType=None, settlDate=None, currency=None,
                 trading_session=None, entries=[], timestamp=None, negotiation_agent=None, **kwargs):
        Event.__init__(self, topic)
        self.mdReqId = mdReqId
        self.symbol = symbol
        self.md_type = md_type
        self.securityId = securityId
        self.securityType = securityType
        self.settlType = settlType
        self.settlDate = settlDate
        self.entries = self.build_entries(entries)
        self.timestamp = timestamp
        self.negotiation_agent = negotiation_agent
        self.trading_session = trading_session
        self.currency = currency

    @staticmethod
    def build_entries(new_entries):
        entries = []
        for entry in new_entries:
            entries.append(MDEntry(**entry))
        return entries

    def get_entry(self, entry_type):
        for entry in self.entries:
            if entry.entry_type == entry_type:
                return entry
        return None

    def addEntry(self, entry):
        self.entries.append(entry)

    def setEntries(self, entries):
        self.entries = entries


class ExecutionReportEvent(Event):
    def __init__(self, topic=EventType.EXECUTION_REPORT_EVENT.value, securityExchange=None, lastQty=None,
                 lastPrice=None, leavesQty=None, execType=None, settlType=None, transactTime=None, timeInForce=None,
                 text=None, symbol=None, side=None, price=None, avgPx=None, orderType=None, orderStatus=None,
                 orderQty=None, orderID=None, execID=None, settlement_date=None, currency=None, cumQty=None,
                 originalClientOrderId=None, account=None, clientOrderID=None, **kwargs):
        Event.__init__(self, topic)
        self.securityExchange = securityExchange
        self.lastQty = lastQty
        self.lastPrice = lastPrice
        self.leavesQty = leavesQty
        self.execType = execType
        self.settlType = settlType
        self.settlement_date = settlement_date
        self.transactTime = transactTime
        self.timeInForce = timeInForce
        self.text = text
        self.symbol = symbol
        self.side = side
        self.price = price
        self.avgPx = avgPx
        self.orderType = orderType
        self.orderStatus = orderStatus
        self.orderQty = orderQty
        self.orderID = orderID
        self.execID = execID
        self.currency = currency
        self.cumQty = cumQty
        self.originalClientOrderId = originalClientOrderId
        self.account = account
        self.clientOrderID = clientOrderID


class NewOrderEvent(Event):
    def __init__(self, topic=EventType.NEW_ORDER_EVENT.value, account=None, cl_ord_id=None, order_qty=None,
                 order_type=None, price=None, side=None, symbol=None, time_in_force=None, currency=None,
                 routing_key=None, exec_inst=None, security_type=None, settlement_type=None, settlement_date=None,
                 execution_id=None, negotiation_agent_id=None, min_qty=None, short_sell=None, **kwargs):
        Event.__init__(self, topic)
        self.account = account
        self.cl_ord_id = cl_ord_id
        self.order_qty = order_qty
        self.order_type = order_type
        self.price = price
        self.side = side
        self.symbol = symbol
        self.time_in_force = time_in_force
        self.routing_key = routing_key
        self.exec_inst = exec_inst
        self.currency = currency
        self.security_type = security_type
        self.settlement_type = settlement_type
        self.settlement_date = settlement_date
        self.execution_id = execution_id
        self.negotiation_agent_id = negotiation_agent_id
        self.min_qty = min_qty
        self.short_sell = short_sell


class OrderCancelReplaceRequestEvent(Event):
    def __init__(self, topic=EventType.ORDER_CANCEL_REPLACE_REQUEST_EVENT.value, account=None, cl_ord_id=None,
                 order_qty=None, order_type=None, orig_client_ord_id=None, orig_order_id=None, price=None, side=None,
                 symbol=None, time_in_force=None, security_type=None, settlement_type=None, currency=None,
                 settlement_date=None, routing_key=None, execution_id=None, negotiation_agent_id=None, short_sell=None,
                 **kwargs):
        Event.__init__(self, topic)
        self.account = account
        self.cl_ord_id = cl_ord_id
        self.order_qty = order_qty
        self.order_type = order_type
        self.orig_client_ord_id = orig_client_ord_id
        self.orig_order_id = orig_order_id
        self.price = price
        self.side = side
        self.symbol = symbol
        self.time_in_force = time_in_force
        self.order_type = order_type
        self.settlement_date = settlement_date
        self.security_type = security_type
        self.settlement_type = settlement_type
        self.routing_key = routing_key
        self.currency = currency
        self.execution_id = execution_id
        self.negotiation_agent_id = negotiation_agent_id
        self.short_sell = short_sell


class BulkNewOrderEvent(Event):
    def __init__(self, topic=EventType.BULK_NEW_ORDER_EVENT.value, orders=[]):
        Event.__init__(self, topic)
        self.orders = orders


class OrderCancelRequestEvent(Event):
    def __init__(self, topic=EventType.ORDER_CANCEL_REQUEST_EVENT.value, account=None, cl_ord_id=None, order_id=None,
                 order_qty=None, orig_cl_ord_id=None, side=None, symbol=None, security_type=None,
                 settlement_type=None, currency=None, settlement_date=None, routing_key=None, execution_id=None,
                 negotiation_agent_id=None, short_sell=None, **kwargs):
        Event.__init__(self, topic)
        self.account = account
        self.cl_ord_id = cl_ord_id
        self.order_id = order_id
        self.order_qty = order_qty
        self.orig_cl_ord_id = orig_cl_ord_id
        self.side = side
        self.symbol = symbol
        self.routing_key = routing_key
        self.security_type = security_type
        self.settlement_type = settlement_type
        self.settlement_date = settlement_date
        self.currency = currency
        self.execution_id = execution_id
        self.negotiation_agent_id = negotiation_agent_id
        self.short_sell = short_sell


class OrderMassCancelRequestEvent(Event):
    def __init__(self, topic=EventType.ORDER_MASS_CANCEL_REQUEST_EVENT.value, account=None, segment=None,
                 clientOrderId=None, **kwargs):
        Event.__init__(self, topic)
        self.account = account
        self.segment = segment
        self.clientOrderId = clientOrderId


class PositionUpdatedEvent(Event):
    def __init__(self, topic=EventType.POSITION_UPDATED_EVENT.value, symbol=None, account=None, size=None, price=None,
                 open_result=None, close_result=None, strategy_id=None, strategy_version=None, execution_id=None,
                 arquants_user=None, execution_type=None):
        Event.__init__(self, topic)
        self.symbol = symbol
        self.account = account
        self.size = size
        self.price = price
        self.open_result = open_result
        self.close_result = close_result
        self.strategy_id = strategy_id
        self.strategy_version = strategy_version
        self.execution_id = execution_id
        self.arquants_user = arquants_user
        self.execution_type = execution_type


class OrderUpdatedEvent(Event):
    def __init__(self, topic=EventType.LT_ORDER_UPDATED_EVENT.value, strategy_id=None, strategy_version=None,
                 execution_id=None, arquants_user=None, execution_type=None, account=None, symbol=None, side=None,
                 orderStatus=None, price=None, orderQty=None, cumQty=None, orderID=None, clientOrderID=None,
                 transactTime=None, ers=[], timeInForce=None, text=None, securityExchange=None, leavesQty=None,
                 execType=None, execID=None, **kwargs):
        Event.__init__(self, topic)
        self.strategy_id = strategy_id
        self.strategy_version = strategy_version
        self.execution_id = execution_id
        self.arquants_user = arquants_user
        self.execution_type = execution_type
        self.account = account
        self.symbol = symbol
        self.side = side
        self.orderStatus = orderStatus
        self.price = price
        self.orderQty = orderQty
        self.cumQty = cumQty
        self.orderID = orderID
        self.clientOrderID = clientOrderID
        self.transactTime = transactTime
        self.execID = execID
        self.execType = execType
        self.leavesQty = leavesQty
        self.securityExchange = securityExchange
        self.text = text
        self.timeInForce = timeInForce
        self.ers = ers
        self.time = str(datetime.now())


class LogsEvent(Event):
    def __init__(self, topic=EventType.LOGS_EVENT.value, arquants_user=None, execution_id=None, execution_type=None,
                 strategy_id=None, log_line=None, **kwargs):
        Event.__init__(self, topic)
        self.arquants_user = arquants_user
        self.execution_id = execution_id
        self.execution_type = execution_type
        self.strategy_id = strategy_id
        self.log_line = log_line


class ParameterChangeEvent(Event):
    def __init__(self, topic=EventType.PARAMETER_CHANGE_EVENT.value, parameters=dict(), execution_id=None,
                 arquants_user=None, strategy_id=None, strategy_version=None, execution_type=None, from_api=False,
                 **kwargs):
        Event.__init__(self, topic)
        self.parameters = parameters
        self.execution_id = execution_id
        self.arquants_user = arquants_user
        self.strategy_id = strategy_id
        self.strategy_version = strategy_version
        self.execution_type = execution_type
        self.from_api = from_api


class ManualOrderCancelRequestEvent(Event):
    def __init__(self, topic=EventType.MANUAL_ORDER_CANCEL_REQUEST_EVENT.value, order_id=None, **kwargs):
        Event.__init__(self, topic)
        self.order_id = order_id


class TradeExecutedEvent(Event):
    def __init__(self, topic=EventType.TRADE_EXECUTED_EVENT.value, strategy_id=None, strategy_version=None,
                 execution_id=None, arquants_user=None, execution_type=None, ts=None, symbol=None, side=None,
                 size=None, price=None, account=None, orderid=None, tradeid=None, settlType=None, **kwargs):
        Event.__init__(self, topic)

        self.strategy_id = strategy_id
        self.strategy_version = strategy_version
        self.execution_id = execution_id
        self.arquants_user = arquants_user
        self.execution_type = execution_type
        self.ts = ts
        self.symbol = symbol
        self.side = side
        self.size = size
        self.price = price
        self.account = account
        self.orderid = orderid
        self.tradeid = tradeid
        self.settlType = settlType


class BuildStrategyErrorEvent(Event):
    def __init__(self, topic=EventType.BUILD_STRATEGY_ERROR_EVENT.value, execution_id=None, arquants_user=None,
                 strategy_id=None, strategy_version=None, execution_type=None, error=None, **kwargs):
        Event.__init__(self, topic)
        self.execution_id = execution_id
        self.arquants_user = arquants_user
        self.strategy_id = strategy_id
        self.strategy_version = strategy_version
        self.execution_type = execution_type
        self.error = error


class LiveTradingStartedEvent(Event):
    def __init__(self, topic=EventType.LIVE_TRADING_STARTED_EVENT.value, execution_id=None, arquants_user=None,
                 status=None, **kwargs):
        Event.__init__(self, topic)
        self.execution_id = execution_id
        self.arquants_user = arquants_user
        self.status = status

class OrderStatusRequestEvent(Event):
    def __init__(self, topic=EventType.ORDER_STATUS_REQUEST_EVENT.value, client_order_id=None, side=None, **kwargs):
        Event.__init__(self, topic)
        self.client_order_id = client_order_id
        self.side = side

class PushNotificationEvent(Event):
    def __init__(self, topic=EventType.PUSH_NOTIFICATION_EVENT.value, title=None, body=None, user=None, device=None,
                 execution_id=None, strategy_id=None):
        Event.__init__(self, topic)
        self.title = title
        self.body = body
        self.user = user
        self.device = device
        self.execution_id = execution_id
        self.strategy_id = strategy_id


class StrategyStartedEvent(Event):
    """
    Evento para el risk module

    """

    def __init__(self, topic=EventType.STRATEGY_STARTED_EVENT.value,
                 execution_id=None,
                 strategy_version=None,
                 rate_limit_orders=10,
                 rate_limit_logs=5,
                 tickers=None,
                 user=None,
                 routing_key=None,
                 type=None,
                 strategy_id=None
                 ):
        Event.__init__(self, topic)
        self.strategy_version = strategy_version
        self.user = user
        self.tickers = tickers
        self.rate_limit_logs = rate_limit_logs
        self.rate_limit_orders = rate_limit_orders
        self.execution_id = execution_id
        self.routing_key = routing_key
        self.type = type
        self.strategy_id = strategy_id


class StrategyStatusChangeEvent(Event):
    """
    Evento enviado por las estrategias, y tambien por la API.

    El evento tiene un significado distinto en cada uno de estos dos casos:
    -Cuando el evento es enviado por la estrategia, se usa para notificar que hubo un cambio de estado en la estrategia.
    -Cuando el evento es enviado por la API, se usa para ordenarle a una estrategia en particular que cambie su estado.

    Posibles valores de estado:
     - ACTIVE
     - STOPPED
     - FINISHED
     - ERROR

    """

    def __init__(self, topic=EventType.STRATEGY_STATUS_CHANGE.value, status=None, from_api=None, execution_id=None):
        Event.__init__(self, topic)
        self.status = status
        self.from_api = from_api
        self.execution_id = execution_id



import threading
import websocket
import simplejson
import datetime
import urllib
import ssl
from queue import Queue

# Cola donde van a ir los mensajes que entren del websocket.
from aq_lib.utils.logger import logger

manager_queue = Queue()


class WebSocketClient:
    def __init__(self, _ws_url, _token=None, ws_uuid=None):
        """
        Params:
            url_base: Url for connect and get token.
            url_ws: Url for websocket connection.
        """
        self.ws_url = _ws_url
        self.token = _token
        self.ws = None
        self.ws_thread = None
        self.active = False
        self.ws_uuid = ws_uuid

    def md_subscribe(self, _payload):
        """
        Me suscribo a un channel del websocket. Por ejemplo marketdata.
        :param _payload: Estructura de "WSMessageMD".
        :return
        """
        data = {'type': 'subscribe', 'payload': _payload}
        self.send_msg(data)

    def md_unsubscribe(self, _payload):
        """
        Me desusbribo a un channel del websocket. Por ejemplo marketdata.
        :param _payload: Estructura de "WSMessageMD".
        :return:
        """
        data = {'type': 'unsubscribe', 'payload': _payload}
        self.send_msg(data)

    def start(self):
        """
        Inicia el websocket, pasandole el querystring como parámetro.
        """
        if not self.active:
            # Check token.
            if self.token is not None:
                query_string = {
                    'token': self.token
                }
                # websocket.enableTrace(True)
                self.ws = websocket.WebSocketApp(self.ws_url + '?' + urllib.parse.urlencode(query_string),
                                                 on_message=lambda ws, msg: self.on_message(ws, msg),
                                                 on_error=lambda ws, msg: self.on_error(ws, msg),
                                                 on_close=lambda ws: self.on_close(ws),
                                                 on_open=lambda ws: self.on_open(ws)
                                                 )
                # sslopt = {"cert_reqs": ssl.CERT_NONE}
                self.ws_thread = threading.Thread(target=self.ws.run_forever,
                                                  kwargs={'ping_interval': 10, 'sslopt': {'cert_reqs': ssl.CERT_NONE}})
                self.ws_thread.start()
                logger.info('Websocket connected! %s ' % self.ws_thread.getName())
                self.active = True
            else:
                logger.info('Websocket cannot connect!')
        else:
            # Websocket is already active.
            logger.info('Websocket is alive!')

    def stop(self):
        """
        Detengo el websocket.
        """
        if self.active:
            self.ws.keep_running = False
            self.active = False
            logger.info('Websocket disconnected!')
        else:
            logger.info('Websocket is not alive!')

    @staticmethod
    def on_message(ws, message):
        """
        Recibo mensaje, evento AQ.
        """
        print(message)
        msg = simplejson.loads(message)
        logger.info(msg)
        # Meto el mensaje en la queue.
        manager_queue.put(msg)

    @staticmethod
    def on_error(ws, error):
        print(error)
        print("### connection error ###")
        print(datetime.datetime.now())
        logger.info('Websocket connection error.')

    @staticmethod
    def on_close(ws):
        print("### connection closed ###")
        print(datetime.datetime.now())
        logger.info('Websocket connection closed.')

    def on_open(self, ws):
        print("Conection Open...")
        print(datetime.datetime.now())
        # Agrego el UUID en la data que envío para crear un mapa del lado de channels #
        self.send_msg({
            'payload': {
                'uuid': self.ws_uuid
            }
        })
        logger.info('Websocket connection opened.')

    def send_msg(self, params):
        """
        Envío mensajes al websocket.
        :param params: dict() con los parametros de la request al ws.
        :return: response
        """
        msg = simplejson.dumps(params)
        self.ws.send(msg)


class WSMessageMD:
    """
    Contiene los campos que tiene un mensaje de websocket.
    :param _market:     Mercado correspondiente de _symbol.
    :param _symbol:     Instrumento a solicitar.
    :param _md_type:    Tipo de marketdata. ('top' | 'aggregated' | 'disaggregated').
    """
    def __init__(self, _market: str, _symbol: str, _md_type: int):
        self.market = _market
        self.symbol = _symbol
        self.md_type = _md_type

    def get_dict(self):
        """
        Retorno dict armado para agregar al payload, y luego al websocket.
        :return dict():
        """
        return self.__dict__


if __name__ == "__main__":
    pass



from enum import Enum


class StrategyStatus(Enum):

    """
    Esto esta duplicado en el modelo de la API. LiveTrading.

    """
    ACTIVE = 'active'
    PAUSED = 'paused'
    FINISHED = 'finished'
    ERROR = 'error'
    INITIALIZING = 'initializing'
    DELETED = 'deleted'



from enum import Enum


class ExecType(Enum):
    New = '0'
    DoneForDay = '3'
    Canceled = '4'
    Replaced = '5'
    PendingCancel = '6'
    Stopped = '7'
    Rejected = '8'
    Suspended = '9'
    PendingNew = 'A'
    Calculated = 'B'
    Expired = 'C'
    Restated = 'D'
    PendingReplace = 'E'
    Trade = 'F'
    TradeCorrect = 'G'
    TradeCancel = 'H'
    OrderStatus = 'I'
    TradeInAClearingHold = 'J'
    TradeHasBeenReleasedToClearing = 'K'
    TriggeredOrActivatedBySystem = 'L'
    Locked = 'M'
    Released = 'N'



class Serializable:
    def __init__(self):
        pass

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}
        for attr, value in self.__dict__.items():
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        return result



from aq_lib.utils.logger import logger
import requests

class HttpClient:
    """
    Cliente HTTP. Se usa para que el "PublicAQHandlerLayer" se comunique con la api.

    :param _username: Nombre de usuario API.
    :param _password: Password de usuario API.
    :param _endpoint: Endpoint a donde van a ir las request de la api.
    """
    def __init__(self, _username, _password, _endpoint, _audience='arquants'):
        self.username = _username
        self.password = _password
        self.endpoint = _endpoint
        self.audience = _audience
        self.get_token()
        self.token = None

    def get_token(self):
        """
        Solicita el token y lo guarda.
        """
        headers = {'Content-Type': 'application/json', 'Authorization': 'my-auth-token'}
        payload = {'username': self.username, 'password': self.password, 'audience': self.audience}
        res = requests.post(self.endpoint + 'auth/login/', json=payload, headers=headers)
        if res.ok:
            self.token = res.json()['token']
            logger.info('(HTTP) Token OK.')
        else:
            print('Error on api get_token.')
            logger.info('(HTTP) Error on api get_token.')

    def get(self, _path, _payload=None):
        """
        Request GET a la API.
        :param _path:       Path de la api.
        :param _payload:    Datos.
        :return:
        """
        headers = {'Content-Type': 'application/json', 'Authorization': 'JWT %s' % self.token}
        res = requests.get(self.endpoint + _path, _payload, headers=headers)
        if res.ok:
            #
            # Manejo la response. Solamente es un confirmación de si la request ingreso ok a la api.
            #
            # return res
            print('response ok.')
            return res
        else:
            print('Error on GET %s, %s' % (self.endpoint + _path, _payload))

    def post(self, _path, event):
        """
        Request POST a la API.
        :param _path: Path de la api.
        :param event:    Evento arquants.
        :return response:
        """
        headers = {'Content-Type': 'application/json', 'Authorization': 'JWT %s' % self.token}
        res = requests.post(self.endpoint + _path, json=event, headers=headers)
        if res.ok:
            #
            # Manejo la response. Solamente es un confirmación de si la request ingreso ok a la api.
            #
            return res
        else:
            print('Error on GET %s, %s' % (self.endpoint + _path, event))


if __name__ == "__main__":
    pass



from configparser import ConfigParser
import os


class Configuration:
    def __init__(self, config_path):
        self.config = ConfigParser()
        self.config.read(config_path)

    def get_property(self, section, key):
        if os.getenv(key, None):
            return os.getenv(key, None)
        try:
            return self.config[section][key]
        except:
            return None






class BaseHandlerLayer:
    """
    Interface para la comunicación entre la librería y el exterior.
    Los objetos que hereden ésta clase, deberán implementar métodos para comunicarse.
    """
    def __init__(self):
        self.consumer = None
        self.producer = None
        self.callbacks = dict() # diccionario con los callback definidos.
        self.public = None

    def send(self, event, *args):
        """
        Envía mensaje al exterior.
        """
        pass

    def recv(self):
        """
        Recibo mensaje desde el exterior.
        """
        pass

    def register_callback(self, topic, method):
        """
        Registra callbacks en un dict().
        """
        pass

    def subscribe(self, market=None, symbol=None, md_type=None, queue_name=None, routing_key=None):
        """
        Reemplaza al método add_queue. Polimorfismo para las clases que heredan.
        """
        pass

    def unsubscribe(self, market=None, symbol=None, md_type=None, queue_name=None, routing_key=None):
        pass


from logging import StreamHandler

from threading import Thread

from multiprocessing import Queue


class AsyncHandlerMixin(object):

    def __init__(self, *args, **kwargs):
        super(AsyncHandlerMixin, self).__init__(*args, **kwargs)
        self.__queue = Queue()
        self.__thread = Thread(target=self.__loop)
        self.__thread.daemon = True
        self.__thread.start()

    def emit(self, record):
        self.__queue.put(record)

    def __loop(self):

        while True:
            record = self.__queue.get()
            try:
                super(AsyncHandlerMixin, self).emit(record)
            except:
                pass


class AsyncStreamHandler(AsyncHandlerMixin, StreamHandler):
    pass


import logging
import os

import sys
from logging.handlers import MemoryHandler

import atexit

from .async_logging_handler import AsyncStreamHandler


def init_logger():
    logger = logging.getLogger("market_connector")

    log_level = os.getenv("LOG_LEVEL", 'NULL')
    logger.setLevel(getattr(logging, log_level, getattr(logging, 'INFO')))

    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(threadName)s - %(message)s')

    ch = AsyncStreamHandler()
    ch.setLevel(logging.DEBUG)
    ch.setFormatter(formatter)

    logger.addHandler(ch)

    return logger


def init_logger_2():
    logger = logging.getLogger("market_connector")

    logLevel = os.getenv("LOG_LEVEL", 'NULL')
    logger.setLevel(getattr(logging, logLevel, getattr(logging, 'INFO')))

    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(threadName)s - %(message)s')

    streamhandler = logging.StreamHandler(sys.stdout)
    streamhandler.setLevel(getattr(logging, logLevel, getattr(logging, 'INFO')))
    streamhandler.setFormatter(formatter)

    memoryhandler = MemoryHandler(capacity=512, flushLevel=logging.ERROR, target=streamhandler)

    def flush():
        memoryhandler.flush()

    atexit.register(flush)

    logger.addHandler(memoryhandler)

    return logger

logger = init_logger()


from enum import Enum


class EventType(Enum):
    MARKET_DATA_REQUEST_EVENT = 'market_data_request_event'
    MARKET_DATA_SNAPSHOT_FULL_REFRESH_EVENT = 'market_data_snapshot_full_refresh_event'
    EXECUTION_REPORT_EVENT = 'execution_report_event'
    NEW_ORDER_EVENT = 'new_order_event'
    ORDER_CANCEL_REQUEST_EVENT = 'order_cancel_request_event'
    ORDER_MASS_CANCEL_REQUEST_EVENT = 'order_mass_cancel_request_event'
    ORDER_CANCEL_REPLACE_REQUEST_EVENT = 'order_cancel_replace_request_event'
    BUILD_STRATEGY_ERROR_EVENT = 'build_strategy_error_event'
    LT_ORDER_UPDATED_EVENT = 'live_trading_order_updated_event'
    LOGS_EVENT = 'logs_event'
    PARAMETER_CHANGE_EVENT = 'parameter_change_event'
    MANUAL_ORDER_CANCEL_REQUEST_EVENT = 'manual_order_cancel_request_event'
    TRADE_EXECUTED_EVENT = 'trade_executed_event'
    LIVE_TRADING_STARTED_EVENT = 'live_trading_started_event'
    BULK_NEW_ORDER_EVENT = 'bulk_new_order_event'
    PUSH_NOTIFICATION_EVENT = 'push_notification_event'
    POSITION_UPDATED_EVENT = 'position_updated_event'
    STRATEGY_STARTED_EVENT = 'strategy_started_event'
    STRATEGY_STATUS_CHANGE = 'strategy_status_change'
    ORDER_STATUS_REQUEST_EVENT = 'order_status_request_event'



from aq_lib.event.event import *


class EventMapper(dict):
    def __init__(self, *args, **kwargs):

        super(dict, self).__init__(*args, **kwargs)

        self[EventType.MARKET_DATA_REQUEST_EVENT.value] = MarketDataRequestEvent
        self[EventType.MARKET_DATA_SNAPSHOT_FULL_REFRESH_EVENT.value] = MarketDataSnapshotFullRefreshEvent
        self[EventType.EXECUTION_REPORT_EVENT.value] = ExecutionReportEvent
        self[EventType.NEW_ORDER_EVENT.value] = NewOrderEvent
        self[EventType.ORDER_CANCEL_REQUEST_EVENT.value] = OrderCancelRequestEvent
        self[EventType.ORDER_MASS_CANCEL_REQUEST_EVENT.value] = OrderMassCancelRequestEvent
        self[EventType.ORDER_CANCEL_REPLACE_REQUEST_EVENT.value] = OrderCancelReplaceRequestEvent
        self[EventType.LT_ORDER_UPDATED_EVENT.value] = OrderUpdatedEvent
        self[EventType.LOGS_EVENT.value] = LogsEvent
        self[EventType.PARAMETER_CHANGE_EVENT.value] = ParameterChangeEvent
        self[EventType.MANUAL_ORDER_CANCEL_REQUEST_EVENT.value] = ManualOrderCancelRequestEvent
        self[EventType.TRADE_EXECUTED_EVENT.value] = TradeExecutedEvent
        self[EventType.BUILD_STRATEGY_ERROR_EVENT.value] = BuildStrategyErrorEvent
        self[EventType.LIVE_TRADING_STARTED_EVENT.value] = LiveTradingStartedEvent
        self[EventType.BULK_NEW_ORDER_EVENT.value] = BulkNewOrderEvent
        self[EventType.PUSH_NOTIFICATION_EVENT.value] = PushNotificationEvent
        self[EventType.POSITION_UPDATED_EVENT.value] = PositionUpdatedEvent
        self[EventType.STRATEGY_STARTED_EVENT.value] = StrategyStartedEvent
        self[EventType.STRATEGY_STATUS_CHANGE.value] = StrategyStatusChangeEvent
        self[EventType.ORDER_STATUS_REQUEST_EVENT.value] = OrderStatusRequestEvent





import setuptools

try: # for pip >= 10
    from pip._internal.req import parse_requirements
    from  pip._internal.network.session import PipSession
except ImportError: # for pip <= 9.0.3
    from pip.req import parse_requirements
    from pip.download import PipSession

from aq_lib import __version__

from setuptools import setup
import os
import re

requires = [
    "amqp==2.2.2",
    "enum34==1.1.6",
    "jsonpickle==0.9.5",
    "kombu==4.2.1",
    "vine==1.1.4",
    "redis==2.10.6",
    "requests==2.22.0",
]


setup(name='aq_lib',
      version=__version__,
      description='public lib for arquants',
      url='https://gitlab.com/arquants/aq-lib.git',
      author='ArQuants',
      author_email='info@arquants.trading',
      license='MIT',
      packages=setuptools.find_packages(),
      install_requires=requires,
      zip_safe=False
)



from enum import Enum


class OrderStatus(Enum):
    New = '0'
    PartiallyFilled = '1'
    Filled = '2'
    DoneForDay = '3'
    Canceled = '4'
    Replaced = '5'
    PendingCancel = '6'
    Stopped = '7'
    Rejected = '8'
    Suspended = '9'
    PendingNew = 'A'
    Calculated = 'B'
    Expired = 'C'
    AcceptedForBidding = 'D'
    PendingReplace = 'E'



from aq_lib.event.event_mapper import EventMapper
from aq_lib.event.event_type import EventType
from aq_lib.manager.handler import BaseHandlerLayer
from aq_lib.manager.wsclient import WebSocketClient
from threading import Thread
import json
import uuid

from aq_lib.manager.httpclient import HttpClient
from aq_lib.utils.logger import logger


class PublicAQHandlerLayer(BaseHandlerLayer):
    """
    Estrategia que implementa a BaseHandlerLayer para interactuar con entorno Demo.
    """
    def __init__(self, _aq_username, _aq_password, _api_endpoint, _ws_endpoint_er, _ws_endpoint_md, _queue):
        BaseHandlerLayer.__init__(self)
        # Diccionario con los endpoints de la api.
        self.endpoint_dict = {
            # EventType.MARKET_DATA_REQUEST_EVENT.value: 'market_data_request',
            EventType.NEW_ORDER_EVENT.value: 'new_order/',
            EventType.BULK_NEW_ORDER_EVENT.value: 'bulk_new_order/',
            EventType.ORDER_CANCEL_REQUEST_EVENT.value: 'order_cancel_request/',
            EventType.ORDER_CANCEL_REPLACE_REQUEST_EVENT.value: 'order_cancel_replace_request/',
            EventType.ORDER_MASS_CANCEL_REQUEST_EVENT.value: 'order_mass_cancel_request/',
            EventType.ORDER_STATUS_REQUEST_EVENT: 'oder_status_request/',
            EventType.TRADE_EXECUTED_EVENT.value: 'trade_execute'
        }
        self.public = True
        self.queue = _queue
        self.http_client = HttpClient(_aq_username, _aq_password, _api_endpoint)
        self.ws_uuid = uuid.uuid4().hex[:16]
        self.ws_er_client = WebSocketClient(_ws_endpoint_er, ws_uuid=self.ws_uuid)
        self.ws_md_client = WebSocketClient(_ws_endpoint_md, ws_uuid=self.ws_uuid)

        self.param_account_id = None
        self.param_negagent_id = None
        self.account = None

        self.active = False
        self.t = None

        # Ésto inicia la clase una vez instanciada.
        # Comentar en caso de querer hacerlo por afuera.
        # self.init_public_aq()

    def init_public_aq(self):
        """
        Inicializo el Public Handler.
        """
        self.init_http()
        self.ws_md_client.token = self.http_client.token
        self.ws_er_client.token = self.http_client.token
        self.init_all_ws()
        # Creo hilo para que el handler empieze a recibir los mensajes de forma asíncrona.
        self.init_thread()
        self._set_account()

    def stop_public_aq(self):
        """
        Finalizo el Public Handler.
        """
        self.active = False
        # self.stop_http()
        self.stop_all_ws()

    def init_all_ws(self):
        """
        Inicio el ws.
        :return:
        """
        self.ws_md_client.start()
        self.ws_er_client.start()

    def stop_all_ws(self):
        """
        Detengo el ws.
        :return:
        """
        self.ws_md_client.stop()
        self.ws_er_client.stop()

    def init_http(self):
        """
        Inicio el http client.
        :return:
        """
        self.http_client.get_token()

    def init_thread(self):
        """
        Inicio un hilo para el método self.recv()
        :return:
        """
        if not self.active:
            self.active = True
            self.t = Thread(target=self.recv)
            self.t.start()
            logger.info('Manager started. %s ' % self.t.getName())

    def send(self, event, *args):
        """
        Según el event.topic, mapear a un endpoint.
        :param event:     event
        :param args:      No se usa, está solamente para completar (polimorfismo).
        :return:
        """
        marketdata_type = {
            '0': 'aggregated',
            '1': 'disaggregated',
            '2': 'top'
        }
        # Lo vuelvo a convertir a dict porque necesito saber su topic.
        event = json.loads(event)

        # Me el tipo de topic. Si es market_data_request, mando un mensaje de suscripcion.
        if event['topic'] == 'market_data_request_event':
            md_type = marketdata_type[str(event['md_type'])]
            for symbol in event['symbols']:
                self.subscribe(market=event['market'], symbol=symbol, md_type=md_type)
        else:
            # Agrego el uuid al evento, para los mensajes de order routing.
            event.update({
                'uuid': self.ws_uuid,
                'param_account_id': self.param_account_id,
                'param_negagent_id': self.param_negagent_id
            })
            if event['topic'] in self.endpoint_dict.keys():
                endpoint = self.endpoint_dict[event['topic']]
                response = self.http_client.post(endpoint, event)
                logger.info(response)
            else:
                logger.info('Not relevant event: %s ' % event['topic'])

    def recv(self):
        """
        Recibo mensaje desde el exterior.
        Lee mensajes desde una queue instanciada.
        """
        while self.active:
            event = self.queue.get()['payload']
            self.process_message(event)

    def subscribe(self, market=None, symbol=None, md_type=None, queue_name=None, routing_key=None):
        """
        Reemplaza al método add_queue.
        queue_name y routing_key no se usan en éste método.

        :param market:      Nombre del mercado.
        :param symbol:      Ticker del Instrumento.
        :param md_type:     Tipo de marketdata (top | aggregated | disaggregated)
        :param queue_name:  [No Usar].
        :param routing_key: [No Usar].
        :return:
        """
        # Armo el payload para el websocket.
        payload = {
            'market': market,
            'symbol': symbol,
            'md_type': md_type
        }
        # Envío mensaje de suscripción al websocket.
        self.ws_md_client.md_subscribe(payload)

    def unsubscribe(self, market=None, symbol=None, md_type=None, queue_name=None, routing_key=None):
        """
        Desuscripción de un instrumento dado. Todo: Ver si es necesario..
        :param market:      Nombre del mercado.
        :param symbol:      Ticker del Instrumento.
        :param md_type:     Tipo de marketdata (top | aggregated | disaggregated)
        :param queue_name:  [No Usar].
        :param routing_key: [No Usar].
        :return:
        """
        # Armo el payload para el websocket.
        payload = {
            'market': market,
            'symbol': symbol,
            'md_type': md_type
        }
        # Envío mensaje de suscripción al websocket.
        self.ws_md_client.md_unsubscribe(payload)

    def register_callback(self, topic, method):
        """
        Registra callbacks en un dict().
        :return:
        """
        self.callbacks[topic] = method

    def get_account(self):
        return self.account

    def process(self, evt):
        if self.callbacks.__contains__(evt.topic):
            self.callbacks[evt.topic](evt)
        else:
            print("No callback defined for %s" % evt.topic)

    def process_message(self, evt):
        try:
            if evt is not None:
                logger.info("Event Received %s" % evt)
                event_mapper = EventMapper()
                evt = event_mapper[evt['topic']](**evt)
                self.process(evt)
        except KeyError:
            logger.error(evt)
            logger.exception("Unknow event topic")
        except Exception as e:
            logger.error(str(evt))
            logger.exception(e)
        finally:
            pass

    def _set_account(self):
        system_accounts_reponse = self.http_client.get('system_account/', None)
        system_accounts = json.loads(system_accounts_reponse.text)
        system_accounts = system_accounts.get('results')

        for system_account in system_accounts:
            for negotiation_account in system_account.get('negotiation_accounts'):
                if negotiation_account.get('negotiation_agent').get('description') == 'Remarket':
                    self.param_account_id = system_account.get('id')
                    self.param_negagent_id = negotiation_account.get('negotiation_agent').get('id')
                    self.account = system_account.get('alias')
                    break
            break
        if self.param_account_id is None or self.param_negagent_id is None:
            raise Exception('Your user cannot run the development lib. Contact support.')



if __name__ == "__main__":
    pass



from aq_lib.transport.serializable import Serializable


class MDEntry(Serializable):
    def __init__(self, entry_type=None, entry_px=0, entry_size=0, trd_type=None, position=None, mdUpdateAction=None):
        self.entry_type = entry_type
        self.entry_px = entry_px
        self.entry_size = entry_size
        self.trd_type = trd_type
        self.position = position
        self.mdUpdateAction = mdUpdateAction
